/*
 * FlexiPix library
 *
 * Copyright 2010-2011 Stefan Schuermans <stefan schuermans info>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef FLP_INC_DISPLAY_H
#define FLP_INC_DISPLAY_H

#include <flexipix/msg.h>
#include <flexipix/types.h>

/**
 * \brief create a new FlexiPix display
 *
 * \param[in] sz_config_file name of config file to read
 * \param[in] p_msg_func message callback function or NULL
 * \param[in] p_msg_ctx user context for message callback
 * \return pointer to new FlexiPix display on success
 *         or NULL on error
 */
flp_display_t *flp_display_create(const char *sz_config_file,
                                  flp_msg_func_p_t p_msg_func,
                                  void *p_msg_ctx);

/**
 * \brief free a FlexiPix display
 *
 * \param[in] p_display pointer to FlexiPix display
 */
void flp_display_free(flp_display_t *p_display);

/**
 * \brief get size of display
 *
 * \param[in] p_display pointer to FlexiPix display
 * \param[out] p_width width of display
 * \param[out] p_height height of display
 */
void flp_display_get_size(flp_display_t *p_display,
                          unsigned int *p_width, unsigned int *p_height);

/**
 * \brief clear image data to output on FlexiPix display
 *
 * clears the stored image data,
 * stored image data can later be sent to the distributors using
 * flp_display_send()
 *
 * \param[in] p_display pointer to FlexiPix display
 */
void flp_display_data_clear(flp_display_t *p_display);

/**
 * \brief set image data to output on FlexiPix display
 *
 * updates (part of) the stored image data,
 * stored image data can later be sent to the distributors using
 * flp_display_send()
 *
 * \param[in] p_display pointer to FlexiPix display
 * \param[in] p_data pointer to rectangular section of image data,
 *                   pixels need to be in R8G8B8 format
 * \param[in] stride_x stride between two pixels in X direction
 * \param[in] stride_y stride between two pixels in Y direction
 * \param[in] x X coordinate of left side of rectangular area
 * \param[in] y Y coordinate of top side of rectangular area
 * \param[in] width with of rectangular area
 * \param[in] height height of rectangular area
 */
void flp_display_data(flp_display_t *p_display, flp_u8_t *p_data,
                      int stride_x, int stride_y,
                      unsigned int x, unsigned int y,
                      unsigned int width, unsigned int height);

/**
 * \brief send image data to distributors
 *
 * sends the currently stored image data to all configured distributors
 *
 * \param[in] p_display pointer to FlexiPix display
 */
void flp_display_send(flp_display_t *p_display);

#endif /* #ifndef FLP_INC_DISPLAY_H */

